﻿// -----------------------------------------------------------------------
// <copyright file="SubscribeHandler.cs" company="Zen Design Corp">
// TODO: Update copyright text.
// </copyright>
// -----------------------------------------------------------------------

namespace Zen.BlackBerry.Subscription.Web
{
	using System;
	using System.Threading.Tasks;
	using System.Web;
	using Microsoft.Practices.ServiceLocation;

	/// <summary>
	/// Responsible for handling a subscribe operation.
	/// </summary>
	/// <remarks>
	/// <para>
	/// The following HTTP request parameters are needed for a resume: appid, address, osversion, model, and type. 
	/// If not using container-managed authentication, the following two HTTP request parameters will also be needed: username, and password.
	/// The type HTTP request parameter is only required if the appid request parameter corresponds to a push application with a type of Public+Enterprise Push.
	/// </para>
	/// <para>
	/// The following return codes can come back from this servlet:
	/// SUCCESS, INVALID_ADDRESS, INVALID_PARAMETERS, INVALID_PUSH_APPLICATION, INVALID_SUBSCRIBER_ID, INVALID_USERNAME, INVALID_PASSWORD, INVALID_TYPE, SUBSCRIPTION_NOT_SUPPORTED, SUBSCRIBER_AUTHENTICATION_FAILED, CONTENT_PROVIDER_SUBSCRIPTION_FAILED, SYSTEM_ERROR
	/// </para>
	/// </remarks>
	public class SubscribeHandler : SubscriptionHandler
	{
		/// <summary>
		/// Gets the type of the subscription operation handled.
		/// </summary>
		/// <value>The type of the subscription.</value>
		public override SubscriptionType SubscriptionType
		{
			get
			{
				return SubscriptionType.Subscribe;
			}
		}

		/// <summary>
		/// Gets the subscription operation details parsed out of the HTTP
		/// request.
		/// </summary>
		/// <param name="context">The context.</param>
		/// <returns>
		/// A <see cref="BaseRequest"/> containing the subscription details.
		/// </returns>
		protected override BaseRequest GetRequest(HttpContext context)
		{
			return SubscriptionRequestParser.ParseSubscribeRequest(context.Request);
		}

		/// <summary>
		/// Asynchronously performs the subscription operation.
		/// </summary>
		/// <param name="context">The context.</param>
		/// <param name="request">The request.</param>
		/// <returns></returns>
		protected override async Task PerformSubscriptionOperation(HttpContext context, BaseRequest request)
		{
			SubscribeRequest subscribeRequest = (SubscribeRequest)request;
			try
			{
				await SubscriptionService.Subscribe(subscribeRequest);
				SetResponse(context.Response, SubscriptionStatusCode.Success);
			}
			catch (Exception)
			{
				SetResponse(context.Response, SubscriptionStatusCode.SystemError);
			}
		}
	}
}
