﻿// -----------------------------------------------------------------------
// <copyright file="PushResponse.cs" company="Zen Design Corp">
// TODO: Update copyright text.
// </copyright>
// -----------------------------------------------------------------------

namespace Zen.BlackBerry.Pap.Parser
{
	using System;
	using System.Xml;
	using System.Xml.Schema;
	using System.Xml.Serialization;
	using Zen.BlackBerry.Common;

	/// <summary>
	/// Represents the response from a push request message.
	/// </summary>
	public class PushResponse : PapMessage, IXmlSerializable
	{
		/// <summary>
		/// Gets or sets the sender address.
		/// </summary>
		/// <value>The sender address.</value>
		public string SenderAddress
		{
			get;
			set;
		}

		/// <summary>
		/// Gets or sets the name of the sender.
		/// </summary>
		/// <value>The name of the sender.</value>
		public string SenderName
		{
			get;
			set;
		}

		/// <summary>
		/// Gets or sets the reply time.
		/// </summary>
		/// <value>The reply time.</value>
		public DateTime ReplyTime
		{
			get;
			set;
		}

		/// <summary>
		/// Gets or sets the code.
		/// </summary>
		/// <value>The code.</value>
		public StatusCode Code
		{
			get;
			set;
		}

		/// <summary>
		/// Gets or sets the description.
		/// </summary>
		/// <value>The description.</value>
		public string Description
		{
			get;
			set;
		}

		#region IXmlSerializable Members

		/// <summary>
		/// This method is reserved and should not be used. When implementing the IXmlSerializable interface, you should return null (Nothing in Visual Basic) from this method, and instead, if specifying a custom schema is required, apply the <see cref="T:System.Xml.Serialization.XmlSchemaProviderAttribute"/> to the class.
		/// </summary>
		/// <returns>
		/// An <see cref="T:System.Xml.Schema.XmlSchema"/> that describes the XML representation of the object that is produced by the <see cref="M:System.Xml.Serialization.IXmlSerializable.WriteXml(System.Xml.XmlWriter)"/> method and consumed by the <see cref="M:System.Xml.Serialization.IXmlSerializable.ReadXml(System.Xml.XmlReader)"/> method.
		/// </returns>
		public XmlSchema GetSchema()
		{
			return null;
		}

		/// <summary>
		/// Generates an object from its XML representation.
		/// </summary>
		/// <param name="reader">The <see cref="T:System.Xml.XmlReader"/> stream from which the object is deserialized.</param>
		public void ReadXml(XmlReader reader)
		{
			PushId = reader["push-id"];
			SenderAddress = reader["sender-address"];
			SenderName = reader["sender-name"];
			ReplyTime = reader.ReadRimTime("reply-time");

			XmlReader subElementReader = reader.ReadSubtree();
			try
			{
				if (subElementReader.ReadToDescendant("response-result"))
				{
					Code = StatusCode.GetCode(subElementReader["code"]);
					Description = subElementReader["desc"];
				}
			}
			finally
			{
				subElementReader.Close();
			}
		}

		/// <summary>
		/// Converts an object into its XML representation.
		/// </summary>
		/// <param name="writer">The <see cref="T:System.Xml.XmlWriter"/> stream to which the object is serialized.</param>
		public void WriteXml(XmlWriter writer)
		{
			writer.WriteAttributeString("sender-address", SenderAddress);
			writer.WriteAttributeString("sender-name", SenderName);
			writer.WriteRimTime("reply-time", ReplyTime);
			writer.WriteStartElement("response-result");
			writer.WriteAttributeString("code", Code.Code);
			writer.WriteAttributeString("desc", Description);
			writer.WriteFullEndElement();
		}

		#endregion
	}
}
